#!/usr/bin/env python

#-----------------------------------------------------------------------
# penny.py
# Author: Bob Dondero
#-----------------------------------------------------------------------

import html
import flask
import database

#-----------------------------------------------------------------------

app = flask.Flask(__name__)

#-----------------------------------------------------------------------

def get_header():

    html_code = ''
    html_code += '<hr>'
    html_code += 'Hello, and welcome to <strong>PennyAdmin</strong>'
    html_code += '<hr>'
    return html_code

#-----------------------------------------------------------------------

def get_footer():

    html_code = ''
    html_code = '<hr>'
    html_code += 'Created by '
    html_code += '<a href="https://www.cs.princeton.edu/~rdondero">'
    html_code += 'Bob Dondero</a>'
    html_code += '<hr>'
    return html_code

#-----------------------------------------------------------------------

@app.route('/', methods=['GET'])
@app.route('/index', methods=['GET'])
def index():

    html_code = ''
    html_code += '<!DOCTYPE html>'
    html_code += '<html>'
    html_code += '<head>'
    html_code += '<title>PennyAdmin</title>'
    html_code += '</head>'
    html_code += '<body>'
    html_code += get_header()
    html_code += '<a href="/show">Show all books</a><br>'
    html_code += '<a href="/add">Add a book</a><br>'
    html_code += '<a href="/delete">Delete a book by ISBN</a><br>'
    html_code += get_footer()
    html_code += '</body>'
    html_code += '</html>'

    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

@app.route('/show', methods=['GET'])
def show():

    books = database.get_books()

    html_code = ''
    html_code += '<!DOCTYPE html>'
    html_code += '<html>'
    html_code += '<head>'
    html_code += '<title>PennyAdmin</title>'
    html_code += '</head>'
    html_code += '<body>'
    html_code += get_header()
    html_code += '<h1>Show All Books</h1>'

    if len(books) == 0:
        html_code += '(None)<br>'
    else:
        pattern = '%s: <strong>%s</strong>: %s<br>'
        for book in books:
            html_code += pattern % (
                html.escape(book['isbn']),
                html.escape(book['author']),
                html.escape(book['title']))

    html_code += '<br>'
    html_code += '<a href="/index">Return to home page</a>'
    html_code += get_footer()
    html_code += '</body>'
    html_code += '</html>'

    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

def report_results(message1, message2):

    html_code = ''
    html_code += '<!DOCTYPE html>'
    html_code += '<html>'
    html_code += '<head>'
    html_code += '<title>PennyAdmin</title>'
    html_code += '</head>'
    html_code += '<body>'
    html_code += get_header()
    html_code += '<h1>Results</h1>'
    html_code += html.escape(message1)
    html_code += '<br>'
    html_code += html.escape(message2)
    html_code += '<br>'
    html_code += '<br>'
    html_code += '<br>'
    html_code += '<a href="/index">Return to home page</a>'
    html_code += get_footer()
    html_code += '</body>'
    html_code += '</html>'

    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

@app.route('/add', methods=['GET'])
def add():

    html_code = ''
    html_code += '<!DOCTYPE html>'
    html_code += '<html>'
    html_code += '<head>'
    html_code += '<title>PennyAdmin</title>'
    html_code += '</head>'
    html_code += '<body>'
    html_code += get_header()

    html_code += '<h1>Add a Book</h1>'
    html_code += '<form action="/handleadd" method="post">'

    html_code += 'Enter an ISBN:'
    html_code += '<input type="text" name="isbn" autofocus '
    html_code += r'required pattern=".*\S.*" '
    html_code += 'title="At least one non-white-space char">'
    html_code += '<br>'

    html_code += 'Enter an author:'
    html_code += '<input type="text" name="author" '
    html_code += r'required pattern=".*\S.*" '
    html_code += 'title="At least one non-white-space char">'
    html_code += '<br>'

    html_code += 'Enter a title:'
    html_code += '<input type="text" name="title" '
    html_code += r'required pattern=".*\S.*" '
    html_code += 'title="At least one non-white-space char">'
    html_code += '<br>'

    html_code += '<input type="submit" value="Go">'

    html_code += '</form>'

    html_code += '<br>'
    html_code += '<a href="/index">Return to home page</a>'
    html_code += get_footer()
    html_code += '</body>'
    html_code += '</html>'

    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

@app.route('/handleadd', methods=['POST'])
def handle_add():

    isbn = flask.request.form.get('isbn')
    if (isbn is None) or (isbn.strip() == ''):
        return report_results('Missing ISBN', '')

    author = flask.request.form.get('author')
    if (author is None) or (author.strip() == ''):
        return report_results('Missing author', '')

    title = flask.request.form.get('title')
    if (title is None) or (title.strip() == ''):
        return report_results('Missing title', '')

    isbn = isbn.strip()
    author = author.strip()
    title = title.strip()

    successful = database.add_book(isbn, author, title)
    if successful:
        message1 = 'The addition was successful'
        message2 = 'The database now contains a book with isbn ' + isbn
        message2 += ' author ' + author + ' and title ' + title
    else:
        message1 = 'The addition was unsuccessful'
        message2 = 'A book with ISBN ' + isbn + ' already exists'

    return report_results(message1, message2)

#-----------------------------------------------------------------------

@app.route('/delete', methods=['GET'])
def delete():

    html_code = ''
    html_code += '<!DOCTYPE html>'
    html_code += '<html>'
    html_code += '<head>'
    html_code += '<title>PennyAdmin</title>'
    html_code += '</head>'
    html_code += '<body>'
    html_code += get_header()

    html_code += '<h1>Delete a Book</h1>'

    html_code += '<form action="/handledelete" method="post">'
    html_code += 'Enter an ISBN:'
    html_code += '<input type="text" name="isbn" autofocus '
    html_code += r'required pattern=".*\S.*" '
    html_code += 'title="At least one non-white-space char"><br>'
    html_code += '<input type="submit" value="Go">'
    html_code += '</form>'

    html_code += '<br>'
    html_code += '<br>'
    html_code += '<a href="/index">Return to home page</a>'
    html_code += get_footer()
    html_code += '</body>'
    html_code += '</html>'

    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

@app.route('/handledelete', methods=['POST'])
def handle_delete():

    isbn = flask.request.form.get('isbn')
    if (isbn is None) or (isbn.strip() == ''):
        return report_results('Missing ISBN', '')

    isbn = isbn.strip()

    database.delete_book(isbn)

    message1 = 'The deletion was successful'
    message2 = 'The database now does not contain a book with ISBN '
    message2 += isbn

    return report_results(message1, message2)
