#!/usr/bin/env python

#-----------------------------------------------------------------------
# penny.py
# Author: Bob Dondero
#-----------------------------------------------------------------------

import os
import flask
import database
import auth

from top import app

#-----------------------------------------------------------------------

@app.route('/', methods=['GET'])
@app.route('/index', methods=['GET'])
def index():

    username = auth.authenticate()
    is_authorized = database.is_authorized(username)

    html_code = flask.render_template('index.html', username=username,
        is_authorized=is_authorized)
    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

@app.route('/show', methods=['GET'])
def show():

    username = auth.authenticate()

    books = database.get_books()
    html_code = flask.render_template('show.html',
        username=username, books=books)
    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

def report_results(username, message1, message2):

    html_code = flask.render_template('reportresults.html',
        username=username, message1=message1, message2=message2)
    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

@app.route('/add', methods=['GET'])
def add():

    username = auth.authenticate()
    if not database.is_authorized(username):
        html_code = 'You are not authorized to add books.'
        response = flask.make_response(html_code)
        return response

    csrf_token = os.urandom(12).hex()
    flask.session['csrf_token'] = csrf_token

    html_code = flask.render_template('add.html',
        username=username, csrf_token=csrf_token)

    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

@app.route('/handleadd', methods=['POST'])
def handle_add():

    username = auth.authenticate()
    if not database.is_authorized(username):
        html_code = 'You are not authorized to add books.'
        response = flask.make_response(html_code)
        return response

    csrf_token_from_session = flask.session.get('csrf_token')
    csrf_token_from_request = flask.request.form.get('csrf_token')

    if ((csrf_token_from_session is None)
        or (csrf_token_from_request is None)
        or (csrf_token_from_request != csrf_token_from_session)):
        html_code = '<title>400 Bad Request</title>'
        html_code += '<h1>Bad Request</h1>'
        html_code += '<p>The CSRF token is missing or bad.</p>'
        response = flask.make_response(html_code)
        return response

    isbn = flask.request.form.get('isbn')
    if (isbn is None) or (isbn.strip() == ''):
        return report_results(username, 'Missing ISBN', '')

    author = flask.request.form.get('author')
    if (author is None) or (author.strip() == ''):
        return report_results(username, 'Missing author', '')

    title = flask.request.form.get('title')
    if (title is None) or (title.strip() == ''):
        return report_results(username, 'Missing title', '')

    isbn = isbn.strip()
    author = author.strip()
    title = title.strip()

    successful = database.add_book(isbn, author, title)
    if successful:
        message1 = 'The addition was successful'
        message2 = 'The database now contains a book with isbn ' + isbn
        message2 += ' author ' + author + ' and title ' + title
    else:
        message1 = 'The addition was unsuccessful'
        message2 = 'A book with ISBN ' + isbn + ' already exists'

    return report_results(username, message1, message2)

#-----------------------------------------------------------------------

@app.route('/delete', methods=['GET'])
def delete():

    username = auth.authenticate()
    if not database.is_authorized(username):
        html_code = 'You are not authorized to delete books.'
        response = flask.make_response(html_code)
        return response

    csrf_token = os.urandom(12).hex()
    flask.session['csrf_token'] = csrf_token

    html_code = flask.render_template('delete.html',
        username=username, csrf_token=csrf_token)

    response = flask.make_response(html_code)
    return response

#-----------------------------------------------------------------------

@app.route('/handledelete', methods=['POST'])
def handle_delete():

    username = auth.authenticate()
    if not database.is_authorized(username):
        html_code = 'You are not authorized to delete books.'
        response = flask.make_response(html_code)
        return response

    csrf_token_from_session = flask.session.get('csrf_token')
    csrf_token_from_request = flask.request.form.get('csrf_token')

    if ((csrf_token_from_session is None)
        or (csrf_token_from_request is None)
        or (csrf_token_from_request != csrf_token_from_session)):
        html_code = '<title>400 Bad Request</title>'
        html_code += '<h1>Bad Request</h1>'
        html_code += '<p>The CSRF token is missing or bad.</p>'
        response = flask.make_response(html_code)
        return response

    isbn = flask.request.form.get('isbn')
    if (isbn is None) or (isbn.strip() == ''):
        return report_results(username, 'Missing ISBN', '')

    isbn = isbn.strip()

    database.delete_book(isbn)

    message1 = 'The deletion was successful'
    message2 = 'The database now does not contain a book with ISBN '
    message2 += isbn

    return report_results(username, message1, message2)
